package cn.com.infosec.crypto.signers;

import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DSA;
import cn.com.infosec.crypto.params.ECKeyParameters;
import cn.com.infosec.crypto.params.ECPrivateKeyParameters;
import cn.com.infosec.crypto.params.ECPublicKeyParameters;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import cn.com.infosec.math.ec.ECAlgorithms;
import cn.com.infosec.math.ec.ECConstants;
import java.math.BigInteger;
import java.security.SecureRandom;

public class ECDSASigner implements ECConstants, DSA {
   ECKeyParameters key;
   SecureRandom random;

   private BigInteger calculateE(BigInteger n, byte[] message) {
      if (n.bitLength() > message.length * 8) {
         return new BigInteger(1, message);
      } else {
         int var3;
         byte[] n1 = new byte[var3 = n.bitLength() / 8];
         System.arraycopy(message, 0, n1, 0, var3);
         return new BigInteger(1, n1);
      }
   }

   public void init(boolean forSigning, CipherParameters param) {
      if (forSigning) {
         if (param instanceof ParametersWithRandom) {
            ECDSASigner var10000 = this;
            ParametersWithRandom var3;
            this.random = (var3 = (ParametersWithRandom)param).getRandom();
            var10000.key = (ECPrivateKeyParameters)var3.getParameters();
         } else {
            this.random = new SecureRandom();
            this.key = (ECPrivateKeyParameters)param;
         }
      } else {
         this.key = (ECPublicKeyParameters)param;
      }

   }

   public BigInteger[] generateSignature(byte[] message) {
      BigInteger var2;
      BigInteger message1 = this.calculateE(var2 = this.key.getParameters().getN(), message);

      BigInteger var4;
      BigInteger var5;
      BigInteger var6;
      BigInteger var8;
      do {
         int var3 = var2.bitLength();

         BigInteger var10000;
         do {
            var10000 = var4 = new BigInteger;
            var10000.<init>(var3, this.random);
         } while(var10000.equals(var5 = ECConstants.ZERO) || (var6 = this.key.getParameters().getG().multiply(var4).getX().toBigInteger().mod(var2)).equals(var5));

         var8 = ((ECPrivateKeyParameters)this.key).getD();
      } while((var8 = var4.modInverse(var2).multiply(message1.add(var8.multiply(var6))).mod(var2)).equals(var5));

      return new BigInteger[]{var6, var8};
   }

   public boolean verifySignature(byte[] message, BigInteger r, BigInteger s) {
      BigInteger var4;
      BigInteger message1 = this.calculateE(var4 = this.key.getParameters().getN(), message);
      BigInteger var5;
      if (r.compareTo(var5 = ECConstants.ONE) >= 0 && r.compareTo(var4) < 0) {
         if (s.compareTo(var5) >= 0 && s.compareTo(var4) < 0) {
            s = message1.multiply(message1 = s.modInverse(var4)).mod(var4);
            message1 = r.multiply(message1).mod(var4);
            return ECAlgorithms.sumOfTwoMultiplies(this.key.getParameters().getG(), s, ((ECPublicKeyParameters)this.key).getQ(), message1).getX().toBigInteger().mod(var4).equals(r);
         } else {
            return false;
         }
      } else {
         return false;
      }
   }
}
